// ------------------------------------------------------------------------------
// DPF.iOS.UIImagePickerController Component
//
// Dadeh Pardazane Faragir ( DPF ) Co.
//
// Web: http://www.dpfaragir.com
//
// Developed By: Babak Yaghoobi
//
// Email #1: yaghoobi@dpfaragir.com
// Email #2: b_yaghobi@yahoo.com
//
// ------------------------------------------------------------------------------
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// ------------------------------------------------------------------------------
unit DPF.iOS.UIImagePickerController;

interface

{$I DPF.iOS.Defs.inc}

uses
  System.SysUtils,
  System.Classes,
  System.Types,
  System.UITypes,
  System.Variants,

  System.TypInfo,
  DPF.iOS.BaseControl,
  DPF.iOS.UIView,
{$IFDEF IOS}
  Macapi.ObjectiveC,
  Macapi.ObjCRuntime,
  IOSapi.CocoaTypes,
  IOSapi.Foundation,
  IOSapi.Uikit,
  IOSapi.CoreGraphics,
  IOSapi.CoreImage,
  FMX.Platform.iOS,
  DPF.iOS.Common,
{$ENDIF}
  FMX.Layouts, FMX.Memo,
{$IFDEF DELPHIXE5} FMX.Graphics, {$ENDIF}FMX.Types, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls;

type

  TDPFUIImagePickerController = class;

  TDPFCameraKind = ( ckBackCamera = 0, ckFrontCamera = 1, ckDefault = 2 );

  // UIImagePickerControllerMediaType
  // UIImagePickerControllerOriginalImage
  // UIImagePickerControllerEditedImage
  // UIImagePickerControllerCropRect
  // UIImagePickerControllerMediaURL
  // UIImagePickerControllerReferenceURL;
  // UIImagePickerControllerMediaMetadata
  TDPFEditingType = ( etOriginalImage, etEditedImage );

  TDPFOndidFinishPickingImage = procedure( Sender: TObject; BitMap: TBitMap ) of object;

{$IFDEF IOS}

  // ------------------------------------------------------------------------------
  UIImagePickerControllerDelegate = interface( IObjectiveC )
    ['{C84AA3F1-67C4-4A7C-B214-7AECAA6F237D}']
    procedure imagePickerController( picker: UIImagePickerController; didFinishPickingMediaWithInfo: NSDictionary ); cdecl;
    procedure imagePickerControllerDidCancel( picker: UIImagePickerController ); cdecl;
  end;

  TDPFImagePickerControllerDelegate = class( TOCLocal, UIImagePickerControllerDelegate )
  private
    FDPFUIImagePickerController: TDPFUIImagePickerController;
  public
    constructor Create( ADPFUIImagePickerController: TDPFUIImagePickerController );
    procedure HidePicker;

    procedure imagePickerController( picker: UIImagePickerController; didFinishPickingMediaWithInfo: NSDictionary ); cdecl;
    procedure imagePickerControllerDidCancel( picker: UIImagePickerController ); cdecl;
  end;

  // ------------------------------------------------------------------------------
{$ENDIF}

  // ------------------------------------------------------------------------------
  [ComponentPlatformsAttribute( PidWin32 or PidWin64 or pidLinux32 or pidAndroid or PidOSX32 or PidiOSDevice or PidiOSSimulator )]
  TDPFUIImagePickerController = class( TComponent )
  private
    FCameraKind             : TDPFCameraKind;
    FOndidFinishPickingImage: TDPFOndidFinishPickingImage;
    FAllowsEditing          : Boolean;
    FShowsCameraControls    : Boolean;
    FEditingType            : TDPFEditingType;
{$IFDEF IOS}
    FDPFImagePickerController: UIImagePickerController;
    FDPFImagePickerControllerDelegate: TDPFImagePickerControllerDelegate;
    FDPFPopover: UIPopoverController;
{$ENDIF}
  protected
    procedure ViewControllerCompletion;
{$IFDEF IOS}
    function TakeImage( AControl: TDPFiOSBaseControl; const ASourceType: UIImagePickerControllerSourceType ): Boolean;
    class function IsAvailableSourceType( const ASourceType: UIImagePickerControllerSourceType ): Boolean;
{$ENDIF}
  public
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;

    function TakeImageFromLibrary( Control: TDPFiOSBaseControl = nil ): Boolean;
    function TakeImageFromCamera( Control: TDPFiOSBaseControl = nil ): Boolean;
  published
    property OndidFinishPickingImage: TDPFOndidFinishPickingImage read FOndidFinishPickingImage write FOndidFinishPickingImage;

    property AllowsEditing      : Boolean read FAllowsEditing write FAllowsEditing default true;
    property ShowsCameraControls: Boolean read FShowsCameraControls write FShowsCameraControls default false;

    property CameraKind : TDPFCameraKind read FCameraKind write FCameraKind default TDPFCameraKind.ckDefault;
    property EditingType: TDPFEditingType read FEditingType write FEditingType default TDPFEditingType.etOriginalImage;
  end;

  // ------------------------------------------------------------------------------

implementation

// ------------------------------------------------------------------------------
{ TDPFUIImagePickerController }
// ------------------------------------------------------------------------------
constructor TDPFUIImagePickerController.Create( AOwner: TComponent );
begin
  inherited Create( AOwner );

  FAllowsEditing       := true;
  FCameraKind          := TDPFCameraKind.ckDefault;
  FEditingType         := TDPFEditingType.etOriginalImage;
  FShowsCameraControls := false;

{$IFDEF IOS}
  FDPFImagePickerControllerDelegate := TDPFImagePickerControllerDelegate.Create( Self );
  FDPFImagePickerController         := TUIImagePickerController.Create;
{$ENDIF}
end;

// ------------------------------------------------------------------------------
destructor TDPFUIImagePickerController.Destroy;
begin
{$IFDEF IOS}
  FDPFImagePickerControllerDelegate.DisposeOf;
  if Assigned( FDPFPopover ) then
    FDPFPopover.release;
  FDPFImagePickerController.release;
  FDPFImagePickerController := nil;
{$ENDIF}
  inherited;
end;

procedure TDPFUIImagePickerController.ViewControllerCompletion;
begin

end;

// ------------------------------------------------------------------------------
{$IFDEF IOS}

function TDPFUIImagePickerController.TakeImage( AControl: TDPFiOSBaseControl; const ASourceType: UIImagePickerControllerSourceType ): Boolean;
var
  frame: CGRect;
  Win  : UIWindow;
begin
  FDPFImagePickerController.setDelegate( FDPFImagePickerControllerDelegate.GetObjectID );
  FDPFImagePickerController.setAllowsEditing( FAllowsEditing );
  FDPFImagePickerController.setAllowsImageEditing( FAllowsEditing );

  Result := IsAvailableSourceType( ASourceType );
  if not Result then
    Exit;

  FDPFImagePickerController.setSourceType( ASourceType );
  if IsAvailableSourceType( UIImagePickerControllerSourceTypeCamera ) and ( ASourceType = UIImagePickerControllerSourceTypeCamera ) then
  begin
    FDPFImagePickerController.setShowsCameraControls( FShowsCameraControls );
    if FCameraKind <> TDPFCameraKind.ckDefault then
      FDPFImagePickerController.setCameraDevice( Integer( FCameraKind ) );
  end;

  Win := GetSharedApplication.keyWindow;

  if not IsIPad or ( ASourceType = UIImagePickerControllerSourceTypeCamera ) then
  begin
    Win.rootViewController.presentModalViewController( FDPFImagePickerController, true );
  end
  else
  begin
    if Assigned( FDPFPopover ) then
    begin
      FDPFPopover.release;
      FDPFPopover := nil;
    end;
    FDPFPopover := TUIPopoverController.Wrap( TUIPopoverController.Alloc.initWithContentViewController( FDPFImagePickerController ) );
    FDPFPopover.retain;

    if Assigned( AControl ) then
      frame := UIView( AControl.UIControl ).bounds
    else
      frame := CGRectMake( 0, 0, 0, 0 );

    FDPFPopover.presentPopoverFromRect( frame, Win.rootViewController.View, UIPopoverArrowDirectionAny, True );
  end;

end;

// ------------------------------------------------------------------------------
class function TDPFUIImagePickerController.IsAvailableSourceType( const ASourceType: UIImagePickerControllerSourceType ): Boolean;
begin
  Result := TUIImagePickerController.OCClass.isSourceTypeAvailable( ASourceType );
end;
{$ENDIF}

// ------------------------------------------------------------------------------
function TDPFUIImagePickerController.TakeImageFromCamera( Control: TDPFiOSBaseControl = nil ): Boolean;
begin
{$IFDEF IOS}
  Result := TakeImage( Control, UIImagePickerControllerSourceTypeCamera );
{$ELSE}
  Result := false;
{$ENDIF}
end;

// ------------------------------------------------------------------------------
function TDPFUIImagePickerController.TakeImageFromLibrary( Control: TDPFiOSBaseControl = nil ): Boolean;
begin
{$IFDEF IOS}
  if IsAvailableSourceType( UIImagePickerControllerSourceTypeCamera ) then
    Result := TakeImage( Control, UIImagePickerControllerSourceTypePhotoLibrary )
  else
    Result := TakeImage( Control, UIImagePickerControllerSourceTypeSavedPhotosAlbum );
{$ELSE}
  Result := false;
{$ENDIF}
end;

{$IFDEF IOS}
// ------------------------------------------------------------------------------
{ TDPFImagePickerControllerDelegate }

constructor TDPFImagePickerControllerDelegate.Create( ADPFUIImagePickerController: TDPFUIImagePickerController );
begin
  inherited create;
  FDPFUIImagePickerController := ADPFUIImagePickerController;
end;

// ------------------------------------------------------------------------------
procedure TDPFImagePickerControllerDelegate.imagePickerController( picker: UIImagePickerController; didFinishPickingMediaWithInfo: NSDictionary );
const
  EType: array [TDPFEditingType] of string = ( 'UIImagePickerControllerOriginalImage', 'UIImagePickerControllerEditedImage' );
var
  img   : UIImage;
  Bitmap: TBitmap;
begin
  if Assigned( FDPFUIImagePickerController.FOndidFinishPickingImage ) then
  begin
    img    := TUIImage.Wrap( didFinishPickingMediaWithInfo.objectForKey( ( CocoaNSStringConst( libKit, EType[FDPFUIImagePickerController.FEditingType] ) as ILocalObject ).GetObjectID ) );
    Bitmap := UIImageToBitmap( img, 0 );
    try
      FDPFUIImagePickerController.FOndidFinishPickingImage( FDPFUIImagePickerController, BitMap );
    finally
      Bitmap.DisposeOf;
    end;
  end;
  HidePicker;
end;

// ------------------------------------------------------------------------------
procedure TDPFImagePickerControllerDelegate.imagePickerControllerDidCancel( picker: UIImagePickerController );
begin
  HidePicker;
end;

// ------------------------------------------------------------------------------
procedure TDPFImagePickerControllerDelegate.HidePicker;
begin
  if not IsIPad or ( FDPFUIImagePickerController.FDPFImagePickerController.sourceType = UIImagePickerControllerSourceTypeCamera ) then
    FDPFUIImagePickerController.FDPFImagePickerController.dismissViewControllerAnimated( true, FDPFUIImagePickerController.ViewControllerCompletion )
  else
  begin
    FDPFUIImagePickerController.FDPFPopover.dismissPopoverAnimated( true );
  end;
end;

// ------------------------------------------------------------------------------
{$ENDIF}

end.
